/***************************************************************************
 * diskio.c - Basic disk I/O routines
 *
 * Copyright (C) 1995 Claus Tondering, ct@login.dknet.dk
 * This file may be redistributed under the terms of the GNU Public License.
 ***************************************************************************/
 
#include <bios.h>
#include <stdlib.h>
#include <stdio.h>
#include "sys/types.h"
#include "part.h"
#include "diskio.h"
#include "e2err.h"
#include "dev.h"

int cyls, heads, sects;  /* sects is sectors per track */
static void diagnostic(int,int,int);
extern struct devdata *init_floppy(int drive ,int sides);
extern int Dstatus;
unsigned long total_sects;

/**********************************************************************
 * getdiskparm finds the physical disk paramters (only called once)
 **********************************************************************/

static int
getdiskparm(int disk)
{
	int i;
	u_char buf[4];
	struct devdata *myddata;

	if(!(disk & 0x80)) {  /* floppy */
		myddata = init_floppy(disk,0); /* Cf. floppy.c */
		if(myddata == NULL)
			return Dstatus;
		sects = myddata->d_sectors;
		heads = myddata->d_heads;
		cyls = myddata->d_tracks;
	}
	else {   /* hard-drive */
		i=biosdisk(8,disk,0,0,0,0,&buf);
		if (i!=0)
			return i;
		sects = buf[0] & 0x3f;
		cyls = ((buf[0] & 0xc0 ) << 2) + buf[1];
		heads = buf[3] + 1;
	}
	total_sects = ((unsigned long)sects)*((unsigned long)cyls)*
			(unsigned long)heads;
	return 0;
}

/**********************************************************************
 * readdisk read a number of 512 byte sectors from the physical disk
 **********************************************************************/

int
readdisk(int disk, unsigned long start, int nsect, void *buf)
{
	unsigned cyl, head, sect; 
	int err;

	if (sects==0) {
		err=getdiskparm(disk);
		if (err) return err;
	}

	if(start > total_sects) 
		return M2E_BAD_SEEK;

	sect = (int)(start % (unsigned long)sects + 1L);
	start /= (unsigned long)sects;
	head = start % heads;
	cyl =  start / heads;

	if((head > 32767) || (cyl > 32767) ||(sect > 32767)) {
		diagnostic(cyl,head,sect);
		return M2E_BAD_SEEK;
	}
        if((unsigned long)head*cyl*sect > total_sects) {
		diagnostic(cyl,head,sect);
		return M2E_BAD_SEEK;
	}

	err = biosdisk(2, disk, head, cyl, sect, nsect, buf);
	if (err==6)
		/* Disk changed - try once more */
		err = biosdisk(2, disk, head, cyl, sect, nsect, buf);

	return err;
}

void
diagnostic(int cyl,int head, int sect)
{
	fprintf(stderr,"Diskio:diagnostic: total_sects = %ul\n\n",total_sects);
	fprintf(stderr,"Disk Geometry as seen by BIOS (C,H,S)=(%d,%d,%d)\n",
			cyls,heads,sects);
	fprintf(stderr,"Request with (C,H,S)=(%d,%d,%d)\n",
			cyl,head,sect);
	return;
}
