/***************************************************************************
 * error code routine
 *
 * Copyright (C) 1995 Claus Tondering, ct@login.dknet.dk
 * This file may be redistributed under the terms of the GNU Public License.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "e2err.h"


struct errlist {
	int value;
	char *text;
} errors[] = {
	ENOENT,		"No such file or directory",
	ENOTDIR,	"Not a directory",
	0x07,		"Illegal disk number",
	0x80,		"Disk not ready",
	0xaa,		"Disk not ready",
	M2E_NOCWD,	"M2CWD environment not specified",
	M2E_BADCWD,	"M2CWD environment has bad format",
	M2E_BADFS,	"Disk does not contain a minix file system",
	M2E_BADPART,"Disk partitioning is corrupted",
	M2E_BADMEM,	"Not enough memory",
	M2E_BAD_SEEK,    "Trying to read beyond limits of disk",
	M2E_BAD_NAME,    "Improper Minix device name format",
	M2E_WHOLE_DISK,  "Access to whole disk devices is not supported",
	M2E_NO_PART,     "No such partition",
	ENOMEM,         "Not enough memory",
};

#define ERRCOUNT (sizeof(errors)/sizeof(errors[0]))



void
e2_err(const char * text, int err)
{
	int i;

	for (i=0; i<ERRCOUNT; i++) {
		if (errors[i].value == err) {
			fprintf(stderr,"%s: %s\n",text,errors[i].text);
			exit(1);
		}
	}

	fprintf(stderr,"%s: Error number %d\n",text,err);
	exit(1);
}

void
e2_err2(const char * text, const char *name, int err)
{
	int i;

	for (i=0; i<ERRCOUNT; i++) {
		if (errors[i].value == err) {
			fprintf(stderr,"%s %s: %s\n", text, name, errors[i].text);
			exit(1);
		}
	}

	fprintf(stderr,"%s %s: Error number %d\n", text, name, err);
	exit(1);
}
