/*  m2cat.c     -- read a file from a Minix file system and print to stdout */
/*  Copyright 1988,1991 Steven W. Harrold - All rights reserved. */
/*  $Header: MREAD.C_V 1.5 91/03/22 07:53:37 SWH Exp $ */


#define USAGE "m2cat [-a] <[drive][:path]>"
#include    <stdio.h>
#include    <ctype.h>
#include    <stdlib.h>
#include    <string.h>
#include    <errno.h>
#include    <dos.h>
#include    "diskio.h"
#include    "part.h"
#include    "genhd.h"
#include    "e2err.h"
#include    "dev.h"
#include    "sys/types.h"
#include    "mfs.h"
#include    "env.h"

unsigned _stklen = 10000;

struct v2_inode *get_inode(struct devdata *ddata, struct super_block *,
                         ino_t *inode, struct v2_inode *ip, char **path);
void minix_to_dos(char text, struct v2_inode *ip, 
                                     struct devdata *ddata, FILE *outfd);
int parse_env_string( char *str, char *dev_name, char *path);
int readenv(char *, char *);


static char buf[BLOCK_SIZE] ;

/*==================================================================*/
main (argc, argv)
int     argc ;
char    *argv[] ;
{
    int     drive ;
    int     partno;
    int     err;
    ino_t     inum;

    long     blkno ;
    struct super_block  sblk ;
    struct part_desc pl[16];

    struct v2_inode  *ip ;
    struct v1_inode *temp;

    long offset;
    struct directory    *darea ;
    struct devdata      my_ddata;
    struct devdata      *ddata ;

    int     i;
    char    *path[MAX_PATH/2+1] ;
    char    text = FALSE ;
    char    *tp ;
    char    device_name[10];
    char    device_name_save[10];
    char    minixfn[MAX_PATH+NAME_SIZE+2] ;
    char    save_minixfn[MAX_PATH+NAME_SIZE+2];

/*  Does (s)he want help?
 */
    if ((argc >= 2) && (argv[1][0] == '-') && (argv[1][1] == '?'))
    {
printf("Copyright 1998 Terry R. McConnell\n");
printf ("Copyright 1988,1991 Steven W. Harrold - All rights reserved\n") ;
        printf ("Version %s\n", VERSION) ;
        printf ("Usage:   m2cat  [-a] [devname:]minixpath \n") ;
        printf ("'-a' translates LF to CRLF, binary otherwise\n") ;
        printf ("'devname' is according to Minix convention, e.g. hda3\n") ;
        printf ("'minixpath' is a fully qualified Minix filename\n") ;
        exit (0) ;
    }

/*  Get the option flag
 */
    if ((argc >= 2) && (argv[1][0] == '-') && (argv[1][1] == 'a'))
    {
        text = TRUE ;
        argc-- ;
        argv++ ;
    }
    else
        text = FALSE ;
   
    if ((argc >= 2) && (argv[1][0] == '-') && (argv[1][1] != 'a')){
	fprintf(stderr,"%s\n",USAGE);
	exit(1);
    }
	
    device_name[0]='\0';
    device_name_save[0]='\0';
    minixfn[0] = '\0';
    readenv(device_name,minixfn);

/*  Fetch the Minix device identifier and path to file
 */
    if (argc != 2)
    {
 	fprintf(stderr,"%s\n",USAGE);
	exit(1);
    }	
    else {
       i = parse_env_string(argv[1],device_name_save,save_minixfn);

       if((i!=NO_DEV)&&(i!=NO_DEV_OR_PATH))
	 strcpy(device_name,device_name_save); /* override dev from env */

       if((i==NO_PATH)||(i==NO_DEV_OR_PATH)){ /* File to copy MUST be given */
 		fprintf(stderr,"%s\n",USAGE);
		exit(1);
       }
       if(save_minixfn[0]!='/') { /* relative path */ 
	   if(*(minixfn+strlen(minixfn)-1)!='/')
		strcat(minixfn,"/");
    	   strcat(minixfn,save_minixfn);
       }
       else strcpy(minixfn,save_minixfn); /* absolute path */
    }

    /* Record final path for use in error messages */
    strcpy(save_minixfn,minixfn);

    /* Read partition tables and fill descriptor array. Convert Device
       identifier name to drive number and partition index */

    name2parms(&drive,&partno,device_name,pl);

    /* Load ddata structure */
    init_fs(drive,partno,pl,&my_ddata);

    ddata = &my_ddata;

/*  Break Minix filename into path components
 */
    if (minixfn[0] != '/')
    {
        fprintf (stderr,
                "You must supply a fully qualified Minix filename\n") ;
        exit (2) ;
    }

    tp = strtok (minixfn, "/") ;
    i = 0 ;
    while (tp)
    {
        path[i++] = tp ;
        tp = strtok (NULL, "/") ;
    }
    path[i] = NULL ;

/*  Read the super block
 */

    offset = ddata->pl[ddata->d_partnum].start; /* offset in sectors of fs
                                                    on disk */
    blkno = 2*SUPER_BLOCK + offset;

    err = readdisk (ddata->d_drive, blkno, 1,buf) ;
    if(err)
        e2_err("m2cp:readdisk",err);

    memcpy (&sblk, buf, sizeof(sblk)) ;

    if (((sblk.s_magic != MINIX_V2_SUPER_MAGIC) && (sblk.s_magic
        != MINIX_V1_SUPER_MAGIC))        ||
        (sblk.s_ninodes < 1)            ||
        (sblk.s_imap_blocks < 1)        ||
        (sblk.s_zmap_blocks < 1))
        e2_err("m2cp",M2E_BADFS);

/*  Store zone to block conversion in dev data struct */
	ddata->d_log_zone_size = sblk.s_log_zone_size;

/*  Locate root directory inode
 */

    inum = 1;
    ip = get_inode(ddata,&sblk, &inum,NULL,NULL); /* 1 is root directory inum */

/*  ip now points to a copy of the root directory inode in memory. We
    pass this to get_inode, which searches for the inode of the desired
    file. If successful, this will return a pointer to an in-memory copy
    of the desired inode.
 */
    ip = get_inode(ddata,&sblk, &inum, ip, path) ;    /* recursively scan */

    if (!ip)
    {
        fprintf (stderr, "Cannot locate Minix file '%s'\n", save_minixfn) ;
        exit (1) ;
    }
    if (!(ip->i_mode & S_IFREG))
    {
        fprintf (stderr, "Minix file '%s' is not a regular file\n",
                         save_minixfn) ;
        exit(1) ;
    }

    minix_to_dos(text, ip, ddata, stdout) ;

/*  Clean up
 */
    free (ip) ;
    return 0;

} /* main() */


#if 0

/* Debugging routine */
int
dump_inode(struct v2_inode *ip){

	int i;
	printf("mode = %u\n",ip->i_mode);
	printf("nlinks = %c\n",'0'+ip->i_nlinks);
	printf("size = %lu\n",ip->i_size);
	for(i=0;i<10;i++)
		printf("zone %d = %lu\n",i,ip->i_zone[i]);
	return 0;
}
#endif

/*---eof---*/
