/*  m2cd.c     -- changes the setting of the default minix device and
                   directory. 
    Copyright 1998 by Terry R. McConnell
*/


#define USAGE "m2cwd <[drive:][path]>"
#include    <stdio.h>
#include    <ctype.h>
#include    <stdlib.h>
#include    <string.h>
#include    <errno.h>
#include    <dos.h>
#include    "diskio.h"
#include    "part.h"
#include    "genhd.h"
#include    "e2err.h"
#include    "dev.h"
#include    "sys/types.h"
#include    "mfs.h"
#include    "env.h"

unsigned _stklen = 5000;

void normalize(char *dir);
struct v2_inode *get_inode(struct devdata *ddata, struct super_block *,
                         ino_t *inode, struct v2_inode *ip, char **path);
int parse_env_string( char *str, char *dev_name, char *path);
int readenv(char *, char *);


static char buf[BLOCK_SIZE] ;

/*==================================================================*/
main (argc, argv)
int     argc ;
char    *argv[] ;
{
    int     drive ;
    int     partno;
    int     err;
    ino_t     inum;

    long     blkno ;
    struct super_block  sblk ;
    struct part_desc pl[16];

    struct v2_inode  *ip ;

    long offset;
    struct directory    *darea ;
    struct devdata      my_ddata;
    struct devdata      *ddata ;

    int     i;
    char    *path[MAX_PATH/2+1] ;
    char    *tp ;
    char    device_name[10];
    char    device_name_save[10];
    char    minixfn[MAX_PATH+NAME_SIZE+2] ;
    char    save_minixfn[MAX_PATH+NAME_SIZE+2];

/*  Does (s)he want help?
 */
    if ((argc >= 2) && (argv[1][0] == '-') && (argv[1][1] == '?'))
    {
	printf("Copyright 1998 Terry R. McConnell\n");
	printf ("Copyright 1988,1991 Steven W. Harrold - All rights reserved\n") ;
        printf ("Version %s\n", VERSION) ;
        printf ("Usage:   m2cwd  [devname:]minixpath \n") ;
        printf ("'devname' is according to Minix convention, e.g. hda3\n") ;
        printf ("'minixpath' is a fully qualified Minix filename\n") ;
        exit (0) ;
    }

	
    device_name[0]='\0';
    device_name_save[0]='\0';
    minixfn[0] = '\0';

/* Get whatever is given in the environment */

    readenv(device_name,minixfn);

/*  Fetch the Minix device identifier and path to file */

    if (argc != 2)
    {
 	fprintf(stderr,"%s\n",USAGE);
	exit(1);
    }	

    i = parse_env_string(argv[1],device_name_save,save_minixfn);

    if((i!=NO_DEV)&&(i!=NO_DEV_OR_PATH))
	 strcpy(device_name,device_name_save); /* override dev from env */
	
    if(save_minixfn[0]!='/') { /* relative path */ 
   	if(*(minixfn+strlen(minixfn)-1)!='/')
		strcat(minixfn,"/");
    	strcat(minixfn,save_minixfn);
    }
    else strcpy(minixfn,save_minixfn); /* absolute path */

    /* Record final path for use in error messages */
    strcpy(save_minixfn,minixfn);

    /* Read partition tables and fill descriptor array. Convert Device
       identifier name to drive number and partition index */

    name2parms(&drive,&partno,device_name,pl);

    /* Load ddata structure */
    init_fs(drive,partno,pl,&my_ddata);

    ddata = &my_ddata;

/*  Break Minix filename into path components
 */
    if (minixfn[0] != '/')
    {
        fprintf (stderr,
                "You must supply a fully qualified Minix filename\n") ;
        exit (2) ;
    }

    tp = strtok (minixfn, "/") ;
    i = 0 ;
    while (tp)
    {
        path[i++] = tp ;
        tp = strtok (NULL, "/") ;
    }
    path[i] = NULL ;

/*  Read the super block
 */

    offset = ddata->pl[ddata->d_partnum].start; /* offset in sectors of fs
                                                    on disk */
    blkno = 2*SUPER_BLOCK + offset;

    err = readdisk (ddata->d_drive, blkno, 1,buf) ;
    if(err)
        e2_err("m2cp:readdisk",err);

    memcpy (&sblk, buf, sizeof(sblk)) ;

    if (((sblk.s_magic != MINIX_V2_SUPER_MAGIC) && (sblk.s_magic
        != MINIX_V1_SUPER_MAGIC))        ||
        (sblk.s_ninodes < 1)            ||
        (sblk.s_imap_blocks < 1)        ||
        (sblk.s_zmap_blocks < 1))
        e2_err("m2cp",M2E_BADFS);

/*  Store zone to block conversion in dev data struct */
	ddata->d_log_zone_size = sblk.s_log_zone_size;

/* If we are changing to a root directory then skip all the searching stuff
   below: we assume each device has a root, and moreover, the search will
   fail because there is no directory named "/" */

    if(strcmp(save_minixfn,"/") != 0) {

    /*  Locate root directory inode */
	    inum = 1;
    	ip = get_inode(ddata,&sblk, &inum,NULL,NULL); /* 1 is root directory inum */

/*  ip now points to a copy of the root directory inode in memory. We
    pass this to get_inode, which searches for the inode of the desired
    directory. If successful, this will return a pointer to an in-memory copy
    of the desired inode.
 */
    	ip = get_inode(ddata,&sblk, &inum, ip, path) ;    /* recursively scan */

    	if (!ip)
    	{
       	 	fprintf (stderr, "No such Minix device or directory '%s'\n", save_minixfn) ;
        	exit (1) ;
    	}
    	if (!(ip->i_mode & S_IFDIR))
    	 {
        	fprintf (stderr, "Minix file '%s' is not a directory\n",
                         save_minixfn) ;
        	exit(1) ;
    	}

    /* At this point we have found the directory we wish to cd to, and it
       is valid. However, the path may contain constructs like .. and .
       which can be collapsed. If not removed, successive uses of this
       program might result in a path that gets too big. */

    	normalize(save_minixfn);
    }

    /* OK, now we just set the M2CWD envar to the directory path, right ???
       NOT! We need to set the variable in the parent process, not in this
       one. This requires some special DOS shenanigans -- in env.c */
    
    strcpy(minixfn,device_name);
    if(minixfn[strlen(minixfn)-1]!=':')
	strcat(minixfn,":");
    strcat(minixfn,save_minixfn);

    set_master_env(ENVAR,minixfn);

    /* Produce some visual effect to make the user feel better */

    printf("cd to %s\n",minixfn);

/*  Clean up */
    free (ip) ;
    return 0;

} /* main() */



/* normalize path: working from left to right in a path apply the translation
   /foo/../ -> /. Then, working from left to right apply the translation
   /./ -> /
*/ 

void
normalize(char *dir)
{
	char *buf1;
	char *buf2;
	char *ptr,*p;

	if(dir == NULL){
		printf("m2cwd:normalize: called with NULL pointer\n");
		exit(1);
	}
	buf1 = (char *)malloc(strlen(dir)+2); 
	if(buf1==NULL)
		e2_err("m2cwd:normalize:", ENOMEM);
	buf2 = (char *)malloc(strlen(dir)+2); 
	if(buf2==NULL)
		e2_err("m2cwd:normalize:", ENOMEM);

	strcpy(buf1,dir);

        /* Make sure the path ends with / */

	if(buf1[strlen(buf1)-1] != '/')
		strcat(buf1,"/");

	/* Squeeze out all /../ */

	while((ptr = strstr(buf1,"/../"))!= NULL){
		if(ptr == buf1){
			printf("m2cwd:normalize: invalid path\n");
			exit(1);
		}
		/* Record where pointer is now */
		p = ptr;
		/* Back pointer up to previous slash */
		while(*--ptr != '/');
		*ptr = '\0';
		strcpy(buf2,buf1);
		strcat(buf2,p+3);
		strcpy(buf1,buf2);
	}

	/* Squeeze out all /./ */

	while((ptr = strstr(buf1,"/./"))!= NULL){
		*ptr = '\0';
		strcpy(buf2,buf1);
		strcat(buf2,ptr+2);
		strcpy(buf1,buf2);
	}
	strcpy(dir,buf1);
	return;
}
