/*  mfs.h       -- data about a Minix file system */
/*  Copyright 1988,1991 Steven W. Harrold. All rights reserved. */
/*  $Header: MFS.H_V 1.7 91/04/05 15:52:01 SWH Exp $ */


#define VERSION "1.01 (January, 1998)"

/* Determine what compiler we are using.  The supported ones are:
**
**      TurboC
**      Microsoft C
*/

#if defined(__TURBOC__)
#define TURBOC
#endif

#if defined(M_I86)
#define MSC
#endif


#define EXTERN      extern  /* used in *.h files */
#define PRIVATE     static  /* PRIVATE x limits the scope of x */
#define PUBLIC              /* PUBLIC is the opposite of PRIVATE */
#define FORWARD             /* some compilers require this to be 'static' */

#define BOOL        int
#define TRUE        1
#define FALSE       0


#define SECTOR_SIZE 512         /* physical sector size in bytes */
#define BLOCK_SIZE  1024        /* # of bytes in a disk block */
#define ZONE_SIZE   7           /* # of direct zones in an inode */
#define V2_NR_TZONES 10         /* Total # of zones in an inode: V2 */
#define V1_NR_TZONES 9          /*  " " ....................."  V1 */
#define NAME_SIZE   14          /* # of bytes in a directory component */
#define MINIX_V1_SUPER_MAGIC 0x137F      /* magic number in super block */
#define MINIX_V2_SUPER_MAGIC 0x2468      /* magic number in super block */
#define MAX_PATH    256         /* max length of path names */
#define MAX_NAMES   128         /* max number of distinct usernames */
#define MAX_GROUPS   64         /* max number of distinct groups */
#define MAX_USER_NAME_LEN 8     /* max number of characters in a username */
#define MAX_GROUP_NAME_LEN 8

#define S_IFMT ((unsigned short)0170000)
#define S_IFREG ((unsigned short)0100000)
#define S_IFBLK 0060000
#define S_IFDIR 0040000
#define S_IFCHR 0020000
#define S_IFIFO 0010000
#define S_ISUID  0004000
#define S_ISGID  0002000
#define S_ISVTX  01000
#define S_IRUSR  00400
#define S_IWUSR  00200
#define S_IXUSR  00100
#define S_IRGRP  00040
#define S_IWGRP  00020
#define S_IXGRP  00010
#define S_IROTH  00004
#define S_IWOTH  00002
#define S_IXOTH  00001
#define S_IFEXEC 0006111

#define S_ISDIR(x) (((x) & S_IFMT) == S_IFDIR)


#define BOOT_BLOCK  0           /* block number of boot block */
#define SUPER_BLOCK 1           /* block number of super block */
#define ROOT_INODE  1           /* inode number for root directory */

typedef unsigned char   byte ;
typedef unsigned short  ushort ;
typedef unsigned long   ulong ;


struct super_block              /* super block, as it is on disk */
{
    ino_t  s_ninodes ;
    zone1_t  s_nzones ;
    short  s_imap_blocks ;
    short  s_zmap_blocks ;
    zone1_t  s_firstdatazone ;
    short  s_log_zone_size ;
    off_t   s_max_size ;
    short  s_magic ;
    short  s_pad;
    zone_t   s_zones;   /* replaces s_nzones in V2 */
} ;

struct v1_inode                    /* an i-node  V1 Filesystem */
{                                 /* As on disk */
    mode_t  i_mode ;
    uid_t   i_uid ;
    off_t   i_size ;
    time_t  i_modtime ;
    gid_t   i_gid ;
    nlink_t i_nlinks ;
    u16_t   i_zone[V1_NR_TZONES-2] ;
    u16_t   i_ind ;
    u16_t  i_dbl_ind ;
} ;

struct v2_inode                    /* an i-node  V2 Filesystem */
{
    mode_t  i_mode ;
    u16_t   i_nlinks;
    u16_t   i_uid ;
    u16_t   i_gid ;
    off_t   i_size ;
    time_t  i_atime; 
    time_t  i_mtime ;
    time_t  i_ctime ;
    zone_t  i_zone[V2_NR_TZONES-3] ;
    zone_t  i_ind ;
    zone_t  i_dbl_ind ;
    zone_t  i_unused;   /* Could be used for triple indirect block */
} ;

struct directory                /* a directory block */
{
    ino_t   d_inum ;            /* 0=empty entry */
    char    d_name[NAME_SIZE] ; /* terminated by \0 char or maxlen */
} ;

extern int
init_fs(int disk, int partno, struct part_desc *pl, struct devdata *dev);

/*---eof---*/
