/*  mmin2dos.c  -- copy a Minix file into a DOS file */
/*  Copyright 1998 Terry R. McConnell  */
/*  Copyright 1988,1991 Steven W. Harrold - All rights reserved. */

#include    <stdio.h>
#include    <stdlib.h>
#include    <errno.h>
#include    "diskio.h"
#include    "e2err.h"
#include    "part.h"
#include    "dev.h"
#include    "sys/types.h"
#include    "mfs.h"

#define     CR      0x0d
#define     LF      0x0a
#define     CTLZ    0x1a

static char buf[BLOCK_SIZE+1] ;

/*==================================================================*/
PRIVATE void mwrite (char *buf, long *flen, int text, FILE *outfd)
{
    int     ll, i ;


    ll = ((*flen > BLOCK_SIZE) ? BLOCK_SIZE : *flen) ;
    *flen -= (long)ll ;

    if (text)
    {
        for (i = 0; i < ll; i++, buf++)
        {
            if (*buf == LF)
                fputc (CR, outfd) ;
            fputc(*buf, outfd) ;
        }
        if ((*flen <= 0)&&(outfd != stdout))
            fputc(CTLZ, outfd) ;
    }

    else
        fwrite (buf, 1, ll, outfd) ;

} /* mwrite() */


/*==================================================================*/
void minix_to_dos(char text, struct v2_inode *ip, 
	struct devdata *ddata, FILE *outfd)
{
    int		v1flag = 0;
    ushort	*ind1_zone;
    ushort      *dbl1_zone;
    byte        *bp ;
    zone_t      *ind_zone;
    zone_t      *dbl_zone;

    int         lim; 
    short       bpz;
    int         i, j, k, l, n , err;
    ulong         blkno ;
    zone_t        zoneno;
    ulong        offset;
    off_t        flen ;

    /* Conversion from zone index to blocks */
    bpz = 1 << (ddata->d_log_zone_size);   /* Blocks per zone */

    /* Reserve memory to store indirect and double indirect zones. */
    /* For version 2 this is way more than enough since we should divide
       by sizeof(ulong) in that case */

     ind_zone = (zone_t *)calloc(sizeof(zone_t),BLOCK_SIZE*bpz/sizeof(zone1_t));
     if(ind_zone == NULL)
	e2_err("min2dos",ENOMEM);
     dbl_zone = (zone_t *)calloc(sizeof(zone_t),BLOCK_SIZE*bpz/sizeof(zone1_t));
     if(dbl_zone == NULL)
	e2_err("min2dos",ENOMEM);

    if(ddata->d_fs_vers == MINIXV2FS)
     lim = BLOCK_SIZE * bpz / sizeof(ulong) ;
    else {
     lim = BLOCK_SIZE * bpz / sizeof(ushort);
     v1flag = 1;
     ind1_zone = (ushort *)ind_zone;
     dbl1_zone = (ushort *)dbl_zone;
    }

/* Offset in sectors of partition from start of disk */
    offset = ddata->pl[ddata->d_partnum].start;

    flen = ip->i_size ;

   /* Read data pointed to by direct zone numbers */
    for (i=0; i<7; i++)
    {
        zoneno = (ip->i_zone)[i] ;
        if (zoneno == 0L)
            break ;
	for(l=0;l<bpz;l++){
           blkno = zoneno*bpz + l;
           err = readdisk(ddata->d_drive, 2*blkno+offset, 2,buf) ;
           if(err)
		e2_err("minix_to_dos:readdisk",err);
          mwrite (buf, &flen, text, outfd) ;
	}
    }

    /* Read data pointed to via the indirect zone number */
    if (ip->i_ind)
    {
        err = readdisk(ddata->d_drive,2*(ip->i_ind)*bpz+offset,2*bpz,ind_zone) ;
        if(err)
		e2_err("minix_to_dos:readdisk",err);

        for (i=0; i<lim; i++)
        {
	    if(v1flag)
              zoneno = ind1_zone[i] ;
	    else 
	      zoneno = ind_zone[i];
            if (zoneno == 0L)
                break ;
	    for(l=0;l<bpz;l++){
		blkno = zoneno*bpz + l;
            	err = readdisk(ddata->d_drive, 2*blkno+offset, 2,buf) ;
            	if(err)
			e2_err("minix_to_dos:readdisk",err);
            	mwrite (buf, &flen, text, outfd) ;
	    }
        }
    }

    /* Read data pointed to via the double indirect zone number */
    if (ip->i_dbl_ind)
    {
        err = readdisk(ddata->d_drive,2*(ip->i_dbl_ind)*bpz+offset,2*bpz,dbl_zone) ;
        if(err)
		e2_err("minix_to_dos:readdisk",err);

        for (i=0; i<lim; i++)
        {
	    if(v1flag)
		zoneno = dbl1_zone[i];
            else 
            	zoneno = dbl_zone[i];
            if (zoneno == 0L)
                break ;
            err = readdisk(ddata->d_drive,2*zoneno*bpz+offset,2*bpz,ind_zone) ;
	    if(err)
		e2_err("minix_to_dos:readdisk",err);
            for (j=0; j<lim; j++)
            {
		if(v1flag)
			zoneno = ind1_zone[j];
		else
                	zoneno = ind_zone[j] ;
                if (zoneno == 0L)
                    break ;
		for(l=0;l<bpz;l++){
			blkno = zoneno*bpz + l;
                	err = readdisk(ddata->d_drive, 2*blkno+offset,2,buf) ;
			if(err)
				e2_err("minix_to_dos:readdisk",err);
                	mwrite (buf, &flen, text, outfd) ;
		}
            }
        }
    }
    /* clean up */
    free(ind_zone);
    free(dbl_zone);
} /* minix_to_dos */


/*---eof---*/
