/*  mrfile.c    -- read a Minix file into a buffer */
/*  Copyright 1998 Terry R. McConnell             */
/*  Copyright 1988,1991 Steven W. Harrold - All rights reserved. */

#include    <stdio.h>
#include    <stdlib.h>
#include    <errno.h>
#include    "part.h"
#include    "e2err.h"
#include    "diskio.h"
#include    "sys/types.h"
#include    "mfs.h"
#include    "dev.h"


/*==================================================================*/
void read_file(void *buf, struct v2_inode *ip, struct devdata *ddata)
{
    byte        *bp ;
    zone_t      *ind_zone;
    zone_t      *dbl_zone;
    ulong	offset;

    int         lim; 
    short	bpz;
    int         i, j, k, l, n ,err;
    ulong         blkno ;
    zone_t         zoneno;

    /* Conversion from zone index to blocks */
    bpz = 1 << (ddata->d_log_zone_size);   /* Blocks per zone */

    /* Reserve memory to store indirect and double indirect zones */
    /* For version 2 this is way more than enough since we should divide
       by sizeof(ulong) in that case. Oh well. Memory is cheap. */

    ind_zone = (zone_t *)calloc(sizeof(zone_t),BLOCK_SIZE*bpz/sizeof(zone1_t));
    if(ind_zone == NULL)
	e2_err("read_file",ENOMEM);
    dbl_zone = (zone_t *)calloc(sizeof(zone_t),BLOCK_SIZE*bpz/sizeof(zone1_t));
    if(dbl_zone == NULL)
	e2_err("read_file",ENOMEM);

    if(ddata->d_fs_vers == MINIXV2FS)
	lim = BLOCK_SIZE * bpz / sizeof(zone_t);
    else
        lim = BLOCK_SIZE * bpz / sizeof(zone1_t);

/* The offset in sectors of the file system from the start of the disk: */
    offset = ddata->pl[ddata->d_partnum].start; 

    /* Read data pointed to by direct zone numbers */
    bp = buf ;
    for (i=0; i<7; i++)
    {
        zoneno = ip->i_zone[i] ;
        if (zoneno == 0L)
            break ;
	for(l=0;l<bpz;l++, bp += BLOCK_SIZE){
	  blkno = zoneno*bpz + l;
	  err = readdisk(ddata->d_drive,2*blkno+offset,2,bp);	
	  if(err)
		e2_err("readfile1",err);
	}
    }

    /* Read data pointed to via the indirect zone number */
    if (ip->i_ind)
    {
	err = readdisk(ddata->d_drive,2*(ip->i_ind)*bpz+offset,2*bpz,ind_zone);
	if(err)
		e2_err("readfile2",err);

        for (i=0; i<lim; i++)
        {
            zoneno = ind_zone[i];
            if (zoneno == 0L)
                break ;
	    for(l=0;l<bpz;l++,bp += BLOCK_SIZE){
	      blkno = zoneno*bpz+l;
	      err = readdisk(ddata->d_drive,2*blkno+offset,2,bp);
	      if(err)
		e2_err("readfile3",err);
	    }
        }
    }

    /* Read data pointed to via the double indirect zone number */
    if (ip->i_dbl_ind)
    {
	err = readdisk(ddata->d_drive,2*(ip->i_dbl_ind)*bpz+offset,2*bpz,dbl_zone);
	if(err)
		e2_err("readfile4",err);

        for (i=0; i<lim; i++)
        {

            zoneno = dbl_zone[i];
            if (zoneno == 0L)
                break ;
	      err = readdisk(ddata->d_drive,2*zoneno*bpz+offset,2*bpz,ind_zone);
	      if(err)
		e2_err("readfile5",err);

              for (j=0; j<lim; j++)
              {
                  zoneno = ind_zone[j];
                  if (zoneno == 0L)
                      break ;
		  for(l=0;l<bpz;l++,bp += BLOCK_SIZE){
		    blkno = zoneno*bpz + l;
	    	    err = readdisk(ddata->d_drive,2*blkno+offset,2,bp);
	            if(err)
		       e2_err("readfile6",err);
		  }
              }
        }
    }
    /* clean up */
    free(ind_zone);
    free(dbl_zone);

} /* read_file */

