/* Build the name of the disk device in Minix convention */

/* We are passed the BIOS disk number, and the count of the partition
   as recorded by the get_part routine.
*/

#include<stdlib.h>
#include<stdio.h>
#include<string.h>
#include "part.h"
#include "e2err.h"

static char Letters[] = "abcd";
extern int sects;
 
int 
build_device_name(int diskno, int partno, char *buf, struct part_desc *pl)
{
	int i=0,j;

	if(diskno < 128){
		sprintf(buf,"fd%d:",diskno);
		return 0;
	}

	if(pl[partno].parent == -1){
		sprintf(buf,"hd%d",(diskno-128)*5 + partno + 1);
		strcat(buf,":");
		return 0;
	}
	/* It's a sub-partition. Have to figure out which letter, a-d */
	i = pl[partno].parent; 
	j = partno;
	while(pl[--j].parent == i);
	sprintf(buf,"hd%d%c:",(diskno-128)*5 + i + 1, Letters[partno-j-1]);
	return 0;
}

/* The "inverse-function" to the one above */
/* Fills the part_desc array using get_part as a side effect. */

int 
name2parms(int *disk, int *partno, char *buf,struct part_desc *pl)
{
	char prefix[3];
	int err;
	int primary;
	char letter;
	int i,j,found;

	switch(strlen(buf)){
		case 3:
			sscanf(buf,"%2s%1d",prefix,&primary);
			if((primary > 9) || (primary < 0)) 
				e2_err("name2parms", M2E_BAD_NAME);

			if(strcmp(prefix,"fd")==0){
				*disk = primary;
				*partno = 0;
				break;
			}
			else if(strcmp(prefix,"hd")==0){
				if((primary == 0) || (primary == 5))
					e2_err("name2parms", M2E_WHOLE_DISK);
				*disk = primary < 5 ? 0x80 : 0x81;
				*partno = primary < 5 ? primary - 1: primary - 6;
				break;
			}
			else e2_err("name2parms", M2E_BAD_NAME);
		case 4:
			sscanf(buf,"%2s%1d%1c",prefix,&primary,&letter);	
			if((primary > 9) || (primary < 0)) e2_err("name2parms", M2E_BAD_NAME);
			if((primary == 5)||(primary==0)) e2_err("name2parms", M2E_WHOLE_DISK); 
			if(strcmp(prefix,"hd")!=0) e2_err("name2parms", M2E_BAD_NAME);
			if(strchr(Letters,letter)==NULL)e2_err("name2parms", M2E_BAD_NAME);
				*disk = primary < 5 ? 0x80 : 0x81;

			/* Need to examine entire partition array to figure
                           out appropriate partno. Simple linear search. */

			sects = 0; /* Force a reread of the disk parameters
                                      in case they have changed */
			err = get_part(*disk,pl);
			if(err)
				e2_err("name2parms:get_part",err);
			found = 0;
			for(i=0;i<16;i++)
				if(pl[i].parttype)
					if(pl[i].parent == primary-1){
						found = 1;
						break;
					}
			if(found) 
				if(pl[i + letter - 'a'].parent == primary-1){
					*partno = i + letter - 'a';
					return 0;
				}
			e2_err("name2parms", M2E_NO_PART);
		default:
			 e2_err("name2parms",M2E_BAD_NAME);
	}
	sects = 0;
	err = get_part(*disk,pl);
	if(err)
		e2_err("name2parms:get_part",err);
	return 0;
}	

#if 0
/* test routine */

#include "sys/types.h"
#include "ldisk.h"

int main(int argc, char **argv)
{
	int partno,drive,disk;
	long offset,length;
	struct part_desc pl[16];
	int err;

	nametodisk(&disk,&offset,&length,pl);
	err = name2parms(&drive,&partno,argv[1],pl);
	if(err)
		e2_err("bzzzzzt...",err);

	printf("drive = %x, partno = %d\n",drive,partno);
	return 0;
}
#endif
