/* passwd.c: support routines for m2ls: read a passwd and group file so
   that real usernames and group names can be given in a long listing.
*/

#include <dir.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "passwd.h"
#include "sys/types.h"
#include "mfs.h"

#define MAXLINE 256

FILE *file_pwd, *file_grp;

/* setpwent: if password file is already open, rewind it. Otherwise, search
   for and open password file. Returns 0 if successful, nonzero if not.
*/

int setpwent(void)
{
	char *p;
	char path[MAX_PATH];

	if(file_pwd){
		rewind(file_pwd);
		return 0;
	}

	/* See if path to passwd file is in environment */

	if((p=getenv(PASSWD_ENV))==NULL)
		if((p = searchpath(DEF_PASSWD))==NULL)
			return E_NOPASS;

	strcpy(path,p);

	if((file_pwd = fopen(path,"r"))==NULL)
		return errno;
	return 0;
}

void
endpwent(void)
{
	fclose(file_pwd);
	return;
}

struct passwd *getpwent(void)
{
	static struct passwd pwd;
	static char name[NAME_MAX];
	static char home_dir[MAX_PATH];
	static char shell[MAX_PATH];
	static int uid;
	static int gid;
	char buffer[MAXLINE];
	char *fld;

	if(file_pwd ==NULL)return NULL;
	if(fgets(buffer,MAXLINE,file_pwd)==NULL) return NULL;

	sscanf(buffer,"%[^:]:%*[^:]:%u:%u:%*[^:]:%[^:]:%s",name,&uid,&gid,home_dir,shell);
	pwd.pw_name = name;
	pwd.pw_uid = uid;
	pwd.pw_gid = gid;
	pwd.pw_dir = home_dir;
	pwd.pw_shell = shell;
	return &pwd;
}


int setgrent(void)
{
	char *p;
	char path[MAX_PATH];

	if(file_grp){
		rewind(file_grp);
		return 0;
	}

	/* See if path to group file is in environment */

	if((p=getenv(GROUP_ENV))==NULL)
		if((p = searchpath(DEF_GROUP))==NULL)
			return E_NOGROUP;

	strcpy(path,p);

	if((file_grp = fopen(path,"r"))==NULL)
		return errno;
	return 0;
}

void
endgrent(void)
{
	fclose(file_grp);
	return;
}

struct group *getgrent(void)
{
	static struct group grp;
	static char name[NAME_MAX];
	static int gid;
	char buffer[MAXLINE];

	if(file_grp ==NULL)return NULL;
	if(fgets(buffer,MAXLINE,file_grp)==NULL) return NULL;

	sscanf(buffer,"%[^:]:%*[^:]:%u",name,&gid);
	grp.gr_name = name;
	grp.gr_gid = gid;
	return &grp;
}

#if 0
/* test main routine */
int
main()
{
	struct passwd *pwd;
	struct group *grp;

	if(setpwent() != 0){
		printf("Can't find passwd file\n");
		return 1;
	}
	
	printf("Password file:\n\n");
	while((pwd = getpwent())!= NULL)
		printf("name = %s, uid = %u\n", pwd->pw_name,pwd->pw_uid);		

	endpwent();

	if(setgrent() != 0){
		printf("Can't find group file\n");
		return 1;
	}
	printf("Group file\n\n");
	while((grp = getgrent())!= NULL)
		printf("name = %s, gid = %u\n", grp->gr_name,grp->gr_gid);		

	endgrent();
	return 0;
}

#endif
